--- old/usr/src/cmd/sgs/libld/common/unwind.c	2015-03-23 21:41:49.218502494 -0400
+++ new/usr/src/cmd/sgs/libld/common/unwind.c	2015-03-23 21:41:49.233935719 -0400
@@ -537,7 +537,8 @@
 	for (APLIST_TRAVERSE(ofl->ofl_unwind, idx, osp)) {
 		uchar_t		*data;
 		size_t		size;
-		uint64_t	off = 0;
+		uint64_t	off = 0, ujunk;
+		int64_t		sjunk;
 		uint_t		cieRflag = 0, ciePflag = 0;
 		Shdr		*shdr;
 
@@ -601,16 +602,40 @@
 				/*
 				 * calign & dalign
 				 */
-				(void) uleb_extract(&data[off], &ndx);
-				(void) sleb_extract(&data[off], &ndx);
+				if (uleb_extract(&data[off], &ndx,
+				    size - off, &ujunk) == DW_OVERFLOW) {
+					ld_eprintf(ofl, ERR_FATAL,
+					    MSG_INTL(MSG_SCN_DWFOVRFLW),
+					    ofl->ofl_name,
+					    osp->os_name);
+					return (S_ERROR);
+				}
+
+				if (sleb_extract(&data[off], &ndx,
+				    size - off, &sjunk) == DW_OVERFLOW) {
+					ld_eprintf(ofl, ERR_FATAL,
+					    MSG_INTL(MSG_SCN_DWFOVRFLW),
+					    ofl->ofl_name,
+					    osp->os_name);
+					return (S_ERROR);
+				}
 
 				/*
 				 * retreg
 				 */
-				if (cieversion == 1)
+				if (cieversion == 1) {
 					ndx++;
-				else
-					(void) uleb_extract(&data[off], &ndx);
+				} else {
+					if (uleb_extract(&data[off], &ndx,
+					    size - off, &ujunk) ==
+					    DW_OVERFLOW) {
+						ld_eprintf(ofl, ERR_FATAL,
+						    MSG_INTL(MSG_SCN_DWFOVRFLW),
+						    ofl->ofl_name,
+						    osp->os_name);
+						return (S_ERROR);
+					}
+				}
 				/*
 				 * we walk through the augmentation
 				 * section now looking for the Rflag
@@ -621,8 +646,15 @@
 					switch (cieaugstr[cieaugndx]) {
 					case 'z':
 					    /* size */
-					    (void) uleb_extract(&data[off],
-						&ndx);
+					    if (uleb_extract(&data[off],
+					        &ndx, size - off, &ujunk) ==
+						DW_OVERFLOW) {
+						ld_eprintf(ofl, ERR_FATAL,
+						    MSG_INTL(MSG_SCN_DWFOVRFLW),
+						    ofl->ofl_name,
+						    osp->os_name);
+						return (S_ERROR);
+					    }
 					    break;
 					case 'P':
 					    /* personality */
@@ -633,11 +665,26 @@
 						 * value to move on to the next
 						 * field.
 						 */
-					    (void) dwarf_ehe_extract(
-						&data[off],
-						&ndx, ciePflag,
+					    switch (dwarf_ehe_extract(
+						&data[off], size - off,
+						&ndx, &ujunk, ciePflag,
 						ofl->ofl_dehdr->e_ident, B_FALSE,
-						shdr->sh_addr, off + ndx, 0);
+						shdr->sh_addr, off + ndx, 0)) {
+					    case DW_OVERFLOW:
+						ld_eprintf(ofl, ERR_FATAL,
+						    MSG_INTL(MSG_SCN_DWFOVRFLW),
+						    ofl->ofl_name,
+						    osp->os_name);
+						return (S_ERROR);
+					    case DW_BAD_ENCODING:
+						ld_eprintf(ofl, ERR_FATAL,
+						    MSG_INTL(MSG_SCN_DWFBADENC),
+						    ofl->ofl_name,
+						    osp->os_name, ciePflag);
+						return (S_ERROR);
+					    case DW_SUCCESS:
+						break;
+					    }
 					    break;
 					case 'R':
 					    /* code encoding */
@@ -661,11 +708,25 @@
 					gotaddr =
 					    ofl->ofl_osgot->os_shdr->sh_addr;
 
-				initloc = dwarf_ehe_extract(&data[off],
-				    &ndx, cieRflag, ofl->ofl_dehdr->e_ident,
-				    B_FALSE,
-				    shdr->sh_addr, off + ndx,
-				    gotaddr);
+				switch (dwarf_ehe_extract(&data[off],
+				    size - off, &ndx, &initloc, cieRflag,
+				    ofl->ofl_dehdr->e_ident, B_FALSE,
+				    shdr->sh_addr, off + ndx, gotaddr)) {
+				case DW_OVERFLOW:
+					ld_eprintf(ofl, ERR_FATAL,
+					    MSG_INTL(MSG_SCN_DWFOVRFLW),
+					    ofl->ofl_name,
+					    osp->os_name);
+					return (S_ERROR);
+				case DW_BAD_ENCODING:
+					ld_eprintf(ofl, ERR_FATAL,
+					    MSG_INTL(MSG_SCN_DWFBADENC),
+					    ofl->ofl_name,
+					    osp->os_name, cieRflag);
+					return (S_ERROR);
+				case DW_SUCCESS:
+					break;
+				}
 
 				/*
 				 * Ignore FDEs with initloc set to 0.
